(*
This script creates a graphical timeline chart for individuals.

Written by Simon Robbins January 2015
*)

property scriptName : "Timeline Report"
property eventOut : "ADOPBAPMBIRTBLESBURICENSCHRxCHRACONFCREMDEATEDUCEMIGEVENFCOMGRADIMMINATINATUOCCUORDNPROBRESIRETIWILLANULDIVxDIVFENGAMARBMARCMARLMARRMARS"
property eventName : {"Adoption", "Baptism", "Birth", "Blessing", "Burial", "Census", "Chritening", "Adult Christening", "Confirmation", "Cremation", "Death", "Education", "Emigration", "Generic Event", "First Communion", "Graduation", "Immigration", "Nationality", "Naturalisation", "Occupation", "Ordination", "Probate", "Residence", "Retirement", "Will", "Annulment", "Divorce", "Divorce Filed", "Engagement", "Marriage Blessing", "Marriage Contract", "Marriage Licence", "Marriage", "Marriage Settlement"}
property MyGlobalUnrpt : {}
property MyGlobalrecs : {}
property MyGlobalfams : {}
property MyGlobalSelRecs : {}
property MyGlobalIndiStrucs : {}
property MyGlobalFamStrucs : {}
property MyGlobaltempstring : {}

global columncount, ChartStart, ChartDays

set MyGlobalUnrpt to {}
set MyGlobalrecs to {}
set MyGlobalfams to {}
set MyGlobalSelRecs to {}
set MyGlobalIndiStrucs to {}
set MyGlobalFamStrucs to {}
set MyGlobaltempstring to {}

-- if no document is open then quit
if CheckAvailable(scriptName) is false then return

set recs to a reference to MyGlobalrecs
set selRecs to a reference to MyGlobalSelRecs

tell application "GEDitCOM II"
	(*Give the user options to choose only individuals with a birth and death date or to include those where one or other is missing. For a small number 
	of selected individuals the difference in speed is minimal but for a large number or the whole database choosing those without a birth or death may
	result in a significant slowing down. This due to the fact that when these are not available the script must cycle through every event for every 
	individual to determine a minimum and maximum event date in order to work out where to put the bar on the chart.
	*)
	set Options to {"Only include individuals with birth and death dates", "Include individuals with no birth or death dates"}
	set thePrompt to "Care: Selecting individuals with no birth and death dates will slow the script significantly as it must cycle through every event to work out when they were alive."
	set r to user choice prompt thePrompt list items Options buttons {"All", "Selected", "Cancel"} title scriptName
	set whichOnes to item 1 of r
	if whichOnes is "Cancel" then return
	set Option to item 1 of item 2 of r
	
	if whichOnes is "All" then
		-- do all individuals
		if Option is "Include individuals with no birth or death dates" then
			set recs to every individual of front document
		else
			tell front document
				set recs to every individual whose (birth SDN is not 0 and death SDN is not 0)
			end tell
		end if
	else
		-- do just the selected records but only if they are individuals
		
		set selRecs to selected records of front document
		set recs to {}
		repeat with thisRec in selRecs
			if record type of thisRec is "INDI" then
				if Option is "Include individuals with no birth or death dates" then
					set end of recs to thisRec
				else
					tell thisRec
						if (birth SDN is not 0 and death SDN is not 0) then
							set end of recs to thisRec
						end if
					end tell
				end if
			end if
		end repeat
	end if
	
	if (count of recs) is 0 then
		display dialog "No selected records were found that could be in the timeline." buttons {"OK"} default button "OK" with title scriptName
		return
	end if
	
	set minbdate to 99999999
	set maxddate to 0
	
	-- First we determine the earliest date of birth and the latest death for all the individuals to calculate the overall width of the chart
	-- If individuals with no date of birth or death have been chosen we get the earliest and latest events to use as a surrogate
	
	
	set fractionStepSize to 0.01 -- progress reporting interval
	set nextFraction to fractionStepSize -- progress reporting interval
	notify progress message "Calculating table width"
	set num to number of items in recs
	repeat with r from 1 to num
		set IndiRec to item r of recs
		tell IndiRec
			set bdate to birth SDN
			set ddate to death SDN
		end tell
		
		if bdate is 0 or ddate is 0 then
			set minedate to 99999999
			set maxedate to 0
			
			set IndiStrucs to (a reference to MyGlobalIndiStrucs)
			set IndiStrucs to every structure in IndiRec
			repeat with thisStruc in IndiStrucs
				
				if eventOut contains name of thisStruc then
					set thisdate to event SDN of thisStruc
					if thisdate is not 0 then
						if thisdate < minedate then
							set minedate to thisdate
						end if
						if thisdate > maxedate then
							set maxedate to thisdate
						end if
					end if
				end if
			end repeat
			set fams to (a reference to MyGlobalfams)
			set fams to (every structure of IndiRec whose name is "FAMS")
			repeat with ThisFam in fams
				set thisfamily to family id (contents of ThisFam) of front document
				set FamStrucs to (a reference to MyGlobalFamStrucs)
				set FamStrucs to every structure in thisfamily
				repeat with thisStruc in FamStrucs
					if eventOut contains name of thisStruc then
						set thisdate to event SDN of thisStruc
						if thisdate is not 0 then
							if thisdate < minedate then
								set minedate to thisdate
							end if
							if thisdate > maxedate then
								set maxedate to thisdate
							end if
						end if
					end if
				end repeat
			end repeat
		end if
		
		if bdate is 0 and minedate is not 99999999 then
			set bdate to minedate
		end if
		
		if ddate is 0 and maxedate is not 0 then
			set ddate to maxedate
		end if
		
		if bdate < minbdate and bdate is not 0 then
			set minbdate to bdate
		end if
		if ddate > maxddate and ddate is not 99999999 then
			set maxddate to ddate
		end if
		
		-- time for progress
		set fractionDone to r / num
		if fractionDone > nextFraction then
			notify progress fraction fractionDone
			set nextFraction to nextFraction + fractionStepSize
		end if
		
	end repeat
	
	if minbdate is 99999999 and maxddate is 0 then
		display dialog "No selected records were found that could be in the timeline." buttons {"OK"} default button "OK" with title scriptName
		return
	end if
	
	-- using the min and max dates found set various variable to define the chart
	set startdate to date text sdn minbdate
	set enddate to date text sdn maxddate
	
	set sdlen to length of startdate
	set edlen to length of enddate
	
	set startyear to text (sdlen - 3) through sdlen of startdate as integer
	set endyear to text (edlen - 3) through edlen of enddate as integer
	
	-- make the table 40 years wider than needed on the left to make room for the names. Also 10 years on the right for the unusual occurance
	-- of someone with no death recorded but another event used as a surrogate near the end of a decade. We need to make sure there is
	-- sufficient space for the fading bar indicating unknown death.
	set startyear to ((round (startyear / 10) rounding down) * 10) - 40
	set endyear to ((round (endyear / 10) rounding down) * 10) + 10
	
	set yearspan to endyear - startyear
	set columncount to (yearspan / 10) + 1
	
	set ChartStart to "1 JAN " & startyear
	set ChartEnd to "31 DEC " & (endyear + 9)
	set ChartStart to sdn range full date ChartStart
	set ChartEnd to sdn range full date ChartEnd
	set ChartDays to (item 1 of ChartEnd) - (item 1 of ChartStart)
	
end tell
-- call method to process the records
tell application "GEDitCOM II"
	set fName to name of front document
	tell front document
		
		set unRpt to my processRecords(recs)
		
	end tell
end tell

-- prepare report

tell application "Finder"
	set screen_size to bounds of window of desktop
	set screen_width to item 3 of screen_size
	set screen_height to item 4 of screen_size
end tell

--Set up the first part of the html file including links to various javascript and css files and set the style of some elements not
-- accounted for in the linked files
set rpt to "<div>" & return
set rpt to rpt & "<!DOCTYPE html>" & return
set rpt to rpt & "<html>" & return
set rpt to rpt & "<head>" & return

-- If the DataTables javascript has not been downloaded, comment out the next two lines and uncomment the two below that
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"/Applications/DataTables-1.10.3/media/css/jquery.dataTables.css\">" & return
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"/Applications/DataTables-1.10.3/extensions/FixedColumns/css/dataTables.fixedColumns.css\">" & return

-- If the DataTables javascript has been downloaded, comment out the next two lines and uncomment the two above
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"https://datatables.net/release-datatables/media/css/jquery.dataTables.css\">" & return
set rpt to rpt & "    <link rel=\"stylesheet\" type=\"text/css\" href=\"https://datatables.net/release-datatables/extensions/FixedColumns/css/dataTables.fixedColumns.css\">" & return

set rpt to rpt & "    <style type=\"text/css\" class=\"init\">" & return
set rpt to rpt & "    /* Ensure that the table scrolls and change some of the settings in the default report.css*/" & return
set rpt to rpt & "    th, td { white-space: nowrap; " & return
set rpt to rpt & "    color:black !important;" & return
set rpt to rpt & "    border-left: 0 !important;" & return
set rpt to rpt & "    border-bottom: 0 !important;" & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    table { border: 0 !important; " & return
set rpt to rpt & "    }" & return
set rpt to rpt & "    div.dataTables_wrapper {" & return
set rpt to rpt & "        width: 100%;" & return
set rpt to rpt & "        margin: 0 auto;" & return
set rpt to rpt & "    }" & return

set rpt to rpt & "*  { font-family: Arial, Helvetica, Geneva, Swiss, SunSans-Regular; font-size: 10pt; }" & return & return

set rpt to rpt & "#details { margin: 0px 0px 0px 0px; border: thin gray solid; padding: 0px 10px 0px 0px; border-collapse:collapse; width: 100%}" & return
set rpt to rpt & "#details th { border: thin gray solid !important; padding: 1px 5px 1px 5px; font-size: x-small; background-color: #ddd; }" & return
set rpt to rpt & "#details td { border: thin gray solid !important; padding: 1px 5px 1px 5px; font-size: x-small; background-color: #f0f0f0;}" & return & return

set rpt to rpt & "th {" & return
set rpt to rpt & "    text-align: left !important;" & return
set rpt to rpt & "    border-left: thin #000000 !important;" & return
set rpt to rpt & "    border-left-style: solid !important;" & return
set rpt to rpt & "    border-bottom-style: none !important;" & return
set rpt to rpt & "    color: #000000 !important; " & return
set rpt to rpt & "    background-color: #ffffff !important; " & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "td {" & return
set rpt to rpt & "    text-align: left !important;" & return
set rpt to rpt & "    border: none !important;" & return
set rpt to rpt & "    color: #000000 !important; " & return
set rpt to rpt & "    background-color: #ffffff !important; " & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "table.dataTable tbody th," & return
set rpt to rpt & "table.dataTable tbody td {" & return
set rpt to rpt & "  padding: 8px 0px;" & return -- This bit gets rid of the padding in the standard datatables css which upsets the positioning of the divs
set rpt to rpt & "}" & return & return

set rpt to rpt & "#Filter {" & return
set rpt to rpt & "    display: none;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & ".bar div { " & return
set rpt to rpt & "    background-color: #004080;" & return
set rpt to rpt & "    border: none;" & return
set rpt to rpt & "    color: white;" & return
set rpt to rpt & "       padding-top: 0;" & return
set rpt to rpt & "    height: 14px;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#key {" & return
set rpt to rpt & "    padding-left: 0;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#ADOP {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: forestgreen;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#BAPM {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#BIRT {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: forestgreen;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#BLES {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#BURI {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: red;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#CENS {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: Chartreuse ;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#CHR {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#CHRA {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#CONF {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#CREM {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: red;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#EMIG {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: goldenrod;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#FCOM {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#GRAD {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: aqua;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#IMMI {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: goldenrod;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#NATU {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: goldenrod;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#OCCU {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: aqua;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#ORDN {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: violet;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#PROB {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: orange;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#RETI {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: aqua;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#WILL {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: orange;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#EVEN {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: aqua;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#DEAT {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: pink;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#ANUL {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#DIV {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#DIVF {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#ENGA {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#MARB {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#MARC {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#MARL {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#MARR {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#MARS {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background-color: gold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#NOBIRT {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background: -webkit-linear-gradient(right, #004080 , white);" & return
set rpt to rpt & "    height: 14px;" & return
set rpt to rpt & "    color: black;" & return
set rpt to rpt & "    font-weight: bold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#NODEAT {" & return
set rpt to rpt & "    border-top: none;" & return
set rpt to rpt & "    background: -webkit-linear-gradient(left, #004080 , white);" & return
set rpt to rpt & "    height: 14px;" & return
set rpt to rpt & "    color: black;" & return
set rpt to rpt & "    font-weight: bold;" & return
set rpt to rpt & "}" & return & return

set rpt to rpt & "#NAME {" & return
set rpt to rpt & "    color: black;" & return
set rpt to rpt & "}" & return & return


set rpt to rpt & "</style>" & return & return

--This bit sets up the DataTables javascript and sets the table so that the columns are not sortable and the initial sort is also suppressed

-- If the DataTables javascript has not been downloaded, comment out the next three lines and uncomment the three below that
set rpt to rpt & "   <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/media/js/jquery.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/media/js/jquery.dataTables.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"/Applications/DataTables-1.10.3/extensions/FixedColumns/js/dataTables.fixedColumns.js\"></script>" & return

-- If the DataTables javascript has  been downloaded, comment out the next three lines and uncomment the three above
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/media/js/jquery.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/media/js/jquery.dataTables.js\"></script>" & return
set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" src=\"https://datatables.net/release-datatables/extensions/FixedColumns/js/dataTables.fixedColumns.js\"></script>" & return

set rpt to rpt & "    <script type=\"text/javascript\" language=\"javascript\" class=\"init\">" & return
set rpt to rpt & "$(document).ready(function() {" & return
set rpt to rpt & "    var table = $('#example').DataTable( {" & return
set rpt to rpt & "        /* Disable initial sort */" & return
set rpt to rpt & "        \"aaSorting\": []," & return
set rpt to rpt & "        \"aoColumns\": [" & return
repeat with c from 1 to (columncount - 1)
	set rpt to rpt & "        { \"bSortable\": false }," & return
end repeat

set rpt to rpt & "{ \"bSortable\": false }" & return
set rpt to rpt & "]," & return
if screen_width < 1600 then
	set rpt to rpt & "        scrollY:        \"" & screen_height - 410 & "px\"," & return
else
	set rpt to rpt & "        scrollY:        \"" & screen_height - 360 & "px\"," & return
end if
set rpt to rpt & "        scrollX:        true," & return
set rpt to rpt & "        scrollCollapse: true," & return
set rpt to rpt & "        paging:         false" & return
set rpt to rpt & "    } );" & return
set rpt to rpt & "" & return
set rpt to rpt & "} );" & return
set rpt to rpt & "    </script>" & return
set rpt to rpt & "</head>" & return & return

set rpt to rpt & "<body>" & return


-- This is for the popup details

-- If the tooltip javascript has not been downloaded, comment out the next line and uncomment the one below
set rpt to rpt & "<script type=\"text/javascript\" src=\"/Applications/wz_tooltip/wz_tooltip.js\"></script>" & return

-- If the tooltip javascript has been downloaded, comment out the next line and uncomment the one above
set rpt to rpt & "<script type=\"text/javascript\" src=\"http://www.walterzorn.de/en/scripts/wz_tooltip.js\"></script>" & return

set rpt to rpt & "    <div width=\"100%\">" & return
set rpt to rpt & "        <section>" & return
set rpt to rpt & "" & return
set rpt to rpt & "" & return

set rpt to rpt & "<h1>Timeline for " & fName & "</h1>" & return & return

-- Next create a key across the top of the report
if screen_width < 1600 then
	set rpt to rpt & "<table id=\"key\">" & return
	set rpt to rpt & "<tr>" & return
	set rpt to rpt & "<td id=\"key\"><b>Key:<b>&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BIRT';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Birth/Adoption&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BAPM';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Baptism/Christening/Blessing/Confirmation/First Communion/Ordination&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='DEAT';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Death&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "</tr><tr><td>&nbsp;</td>"
	set rpt to rpt & "<td id=\"key\"><span id='CENS';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Census&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='IMMI';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Immigration/Emmigration/Naturalisation&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BURI';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Burial/Cremation&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "</tr><tr><td>&nbsp;</td>"
	set rpt to rpt & "<td id=\"key\"><span id='WILL';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Will/Probate&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='MARR';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Marriage/Family Event&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='EVEN';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Other&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "</tr>" & return
	set rpt to rpt & "</table>" & return
else
	set rpt to rpt & "<table id=\"key\">" & return
	set rpt to rpt & "<tr>" & return
	set rpt to rpt & "<td id=\"key\"><b>Key:<b>&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BIRT';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Birth/Adoption&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BAPM';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Baptism/Christening/Blessing/Confirmation/First Communion/Ordination&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='CENS';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Census&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='IMMI';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Immigration/Emmigration/Naturalisation&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='DEAT';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Death&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='BURI';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Burial/Cremation&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='WILL';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Will/Probate&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='EVEN';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Other&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "<td id=\"key\"><span id='MARR';>&nbsp</span></td>" & return
	set rpt to rpt & "<td id=\"key\">Marriage/Family Event&nbsp&nbsp&nbsp</td>" & return
	set rpt to rpt & "</tr>" & return
	set rpt to rpt & "</table>" & return
end if

--Now start the main table of data
set rpt to rpt & "<table id=\"example\" class=\"stripe row-border order-column\" cellspacing=\"0\" width=\"100%\">" & return

set rpt to rpt & "<thead>" & return
set rpt to rpt & "<tr>" & return

--As the table width dynamically sets itself based on the earliest birth date and latest death date we need to set the correct number of columns
repeat with c from 1 to columncount
	set rpt to rpt & "<th style \"align=left\">" & startyear + (c * 10) - 10 & "s</th>" & return
end repeat

set rpt to rpt & "</tr>" & return
set rpt to rpt & "</thead>" & return
set rpt to rpt & "<tbody>" & return

set rpt to rpt & return & (unRpt as string) & return

set rpt to rpt & "</tbody>" & return
set rpt to rpt & "</table>" & return

set rpt to rpt & "</body>" & return
set rpt to rpt & "</html>" & return
-- end the report
set rpt to rpt & "</div>"

-- add report to document and open it
tell application "GEDitCOM II"
	tell front document
		set newreport to make new report with properties {name:"Timeline Report", body:rpt}
		show browser of newreport
	end tell
	
	
	tell application "GEDitCOM II"
		set br to bounds of front window
		set item 1 of br to 22
		set item 3 of br to screen_width - 22
		--end if
		set item 2 of br to 50
		set item 4 of br to screen_height - 40
		
		set bounds of front window to br
	end tell
	
end tell

return



on processRecords(recs)
	tell application "GEDitCOM II"
		set fractionStepSize to 0.01 -- progress reporting interval
		set nextFraction to fractionStepSize -- progress reporting interval
		notify progress message "Gathering individual data"
		-- NOTE: any text from file that might have unsafe html characters (e.g., quotes)
		-- must be processed by safe html before adding to unRpt
		set unRpt to a reference to MyGlobalUnrpt
		set linenum to 0
		set num to number of items in recs
		repeat with r from 1 to num
			set IndiRec to item r of recs
			tell IndiRec
				set bdate to birth SDN
				set ddate to death SDN
				
				
				if bdate is 0 or ddate is 0 then
					set minedate to 99999999
					set maxedate to 0
					set IndiStrucs to every structure in IndiRec
					repeat with thisStruc in IndiStrucs
						if eventOut contains name of thisStruc then
							set thisdate to event SDN of thisStruc
							if thisdate is not 0 then
								if thisdate < minedate then
									set minedate to thisdate
								end if
								if thisdate > maxedate then
									set maxedate to thisdate
								end if
							end if
						end if
					end repeat
					set fams to (every structure of IndiRec whose name is "FAMS")
					repeat with ThisFam in fams
						tell application "GEDitCOM II"
							set thisfamily to family id (contents of ThisFam) of front document
						end tell
						set FamStrucs to every structure in thisfamily
						repeat with thisStruc in FamStrucs
							if eventOut contains name of thisStruc then
								set thisdate to event SDN of thisStruc
								if thisdate is not 0 then
									if thisdate < minedate then
										set minedate to thisdate
									end if
									if thisdate > maxedate then
										set maxedate to thisdate
									end if
								end if
							end if
						end repeat
					end repeat
				end if
				
				if bdate is 0 and minedate is not 99999999 then
					set bdate to minedate
				end if
				
				if ddate is 0 and maxedate is not 0 then
					set ddate to maxedate
				end if
				
				
				if bdate is not 99999999 and ddate is not 0 then
					
					set lifespandays to ddate - bdate
					
					set barlength to (lifespandays / ChartDays) * 100
					set barstart to ((bdate - (item 1 of ChartStart)) / ChartDays) * 100
					
					set RecID to id of IndiRec
					set RecordName to alternate name of IndiRec
					try
						set thisspan to "(" & life span of IndiRec & ")"
					on error
						set thisspan to ""
					end try
					
					set chars to number of characters in (RecordName & " " & thisspan)
					
					set the end of unRpt to "<tr>" & return & return & "<td class='bar';  colspan='" & columncount & "'><div style='width: " & barlength & "%; position: relative; left: " & barstart & "%'>" & return & return
					
					
					try
						set x to event date of (structure named "BIRT")
						if x is not "" then
							set hasBirt to "Y"
						else
							set hasBirt to "N"
						end if
					on error
						set hasBirt to "N"
					end try
					
					try
						set x to event date of (structure named "BAPM")
						if x is not "" then
							set hasBapm to "Y"
						else
							set hasBapm to "N"
						end if
					on error
						set hasBapm to "N"
					end try
					
					try
						set x to event date of (structure named "CHR")
						if x is not "" then
							set hasChr to "Y"
						else
							set hasChr to "N"
						end if
					on error
						set hasChr to "N"
					end try
					
					try
						set x to event date of (structure named "WILL")
						if x is not "" then
							set hasWill to "Y"
						else
							set hasWill to "N"
						end if
					on error
						set hasWill to "N"
					end try
					
					try
						set x to event date of (structure named "DEAT")
						if x is not "" then
							set hasDeat to "Y"
						else
							set hasDeat to "N"
						end if
					on error
						set hasDeat to "N"
					end try
					
					try
						set x to event date of (structure named "BURI")
						if x is not "" then
							set hasBuri to "Y"
						else
							set hasBuri to "N"
						end if
					on error
						set hasBuri to "N"
					end try
					
					try
						set x to event date of (structure named "CREM")
						if x is not "" then
							set hasCrem to "Y"
						else
							set hasCrem to "N"
						end if
					on error
						set hasCrem to "N"
					end try
					
					try
						set x to event date of (structure named "PROB")
						if x is not "" then
							set hasProb to "Y"
						else
							set hasProb to "N"
						end if
					on error
						set hasProb to "N"
					end try
					
					-- safe html change
					set rnAndSpan to safe html raw text (RecordName & " " & thisspan)
					
					if hasBirt is "N" and hasBapm is "N" and hasChr is "N" then
						set end of unRpt to "<span id='NAME'; style=\"position: absolute; left: -" & (chars * 7) + 55 & "px; \"><a href='//" & RecID & "'>" & rnAndSpan & "</a></span>" & return & return & "<span id='NOBIRT'; style=\"position: absolute; left: -50px; \">?&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp</span>" & return & return
					else
						set end of unRpt to "<span id='NAME'; style=\"position: absolute; left: -" & chars * 7 & "px; \"><a href='//" & RecID & "'>" & RecordName & " " & thisspan & "</a></span>" & return & return
					end if
					
					if hasDeat is "N" and hasBuri is "N" and hasCrem is "N" then
						set end of unRpt to "<span id='NODEAT'; style=\"position: absolute; right: -50px; \">&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp?</span>" & return & return
					end if
					
					-- Now deal with the events
					-- Individual events first
					set IndiStrucs to every structure in IndiRec
					my GetEvents(IndiStrucs, sex of IndiRec, bdate, ddate, lifespandays, unRpt, hasBirt, hasBapm, hasChr, hasWill, hasDeat, hasBuri, hasCrem, hasProb)
					
					--Next we do the family events
					set fams to (every structure whose name is "FAMS") of IndiRec
					
					repeat with thisStruc in fams
						set Famid to contents of thisStruc
						tell application "GEDitCOM II"
							set thisfamily to family id Famid of front document
						end tell
						set FamStrucs to every structure in thisfamily
						my GetEvents(FamStrucs, sex of IndiRec, bdate, ddate, lifespandays, unRpt, hasBirt, hasBapm, hasChr, hasWill, hasDeat, hasBuri, hasCrem, hasProb)
					end repeat
				end if
			end tell
			
			set end of unRpt to "</div></td></tr>" & return & return
			
			-- time for progress
			set fractionDone to r / num
			if fractionDone > nextFraction then
				notify progress fraction fractionDone
				set nextFraction to nextFraction + fractionStepSize
			end if
			
		end repeat
	end tell
	return MyGlobalUnrpt
end processRecords


on GetEvents(Structs, sex, bdate, ddate, lifespandays, unRpt, hasBirt, hasBapm, hasChr, hasWill, hasDeat, hasBuri, hasCrem, hasProb)
	tell application "GEDitCOM II"
		
		repeat with thisStruc in Structs
			repeat 1 times
				if eventOut contains name of thisStruc then
					set evnttag to name of thisStruc
					set pos to offset of evnttag in eventOut
					set evntname to item ((((pos) - 1) / 4) + 1) of eventName
					if evntname is "Generic Event" then
						-- look for generic event name
						tell thisStruc
							set genname to contents
							if genname is "" or genname is "Y" then
								set genname to evaluate expression "TYPE"
							end if
						end tell
						if genname is not "" then
							set evntname to genname
						end if
					end if
					set evntdate to (event date of thisStruc)
					if evntdate is "" then
						--we don't have a date so don't display it
						return
					end if
					set evntSDN to (event SDN of thisStruc)
					
					tell thisStruc
						set evntplac to evaluate expression "PLAC"
						set evntage to evaluate expression "AGE"
						set evnttype to evaluate expression "TYPE"
						set evntaddr to evaluate expression "ADDR"
						set evntcaus to evaluate expression "CAUS"
						set evntagnc to evaluate expression "AGNC"
					end tell
					
					--Just in case there were any apostrophes in the data we need to escape them
					
					if evntaddr contains "'" then
						set evntaddr to my findandreplace2(evntaddr, "'", "\\'")
					end if
					
					-- The next four are unlikely, but just in case...
					if evntplac contains "'" then
						set evntplac to my findandreplace2(evntplac, "'", "\\'")
					end if
					
					if evnttype contains "'" then
						set evnttype to my findandreplace2(evnttype, "'", "\\'")
					end if
					
					if evntcaus contains "'" then
						set evntcaus to my findandreplace2(evntcaus, "'", "\\'")
					end if
					
					if evntagnc contains "'" then
						set evntagnc to my findandreplace2(evntagnc, "'", "\\'")
					end if
					
					-- NOTE: any text from file that might have unsafe html characters (e.g., quotes)
					-- must be processed by safe html before adding to unRpt
					tell thisStruc
						set evntname to safe html raw text evntname
						set evntdate to safe html raw text evntdate
						set evntplac to safe html raw text evntplac
						set evntaddr to safe html raw text evntaddr
						if evntaddr is not "" then
							set newText to {}
							set holdDelim to AppleScript's text item delimiters
							set AppleScript's text item delimiters to {return & linefeed, return, linefeed}
							set addrText to text items of evntaddr
							-- If there were any blank lines in the address we need to get 
							-- rid of them otherwise there will be isolated commas in the 
							-- finished text
							repeat with i from 1 to number of items in addrText
								if item i of addrText is not "" then
									set end of newText to item i of addrText
								end if
							end repeat
							set AppleScript's text item delimiters to {", "}
							set evntaddr to newText as text
							set AppleScript's text item delimiters to holdDelim
							
							--Just in case there were any commas already in the data or spaces at the end of a line 
							--we will get double commas so find any and replace them
							set evntaddr to my findandreplace(evntaddr, ",,", ",")
							set evntaddr to my findandreplace(evntaddr, " ,", ",")
						end if
						set evnttype to safe html raw text evnttype
						set evntcaus to safe html raw text evntcaus
						set evntagnc to safe html raw text evntagnc
					end tell
					
					
					
					set popup to "<a href=\"javascript:void(0)\" onmouseover=\"Tip('<table  id=\\'details\\'><tr><th align=\\'right\\' width=\\'45\\'>Event: <\\/th><th align=\\'left\\'>" & evntname & "<\\/th><\\/tr>"
					set popup to popup & "<tr><td align=\\'right\\'>Date: <\\/td><td>" & evntdate & "<\\/td><\\/tr>"
					if evntplac is not "" then
						set popup to popup & "<tr><td align=\\'right\\'>Place: <\\/td><td>" & evntplac & "<\\/td><\\/tr>"
					end if
					if evntaddr is not "" then
						set popup to popup & "<tr><td align=\\'right\\'>Address: <\\/td><td>" & evntaddr & "<\\/td><\\/tr>"
					end if
					if evnttype is not "" then
						set popup to popup & "<tr><td align=\\'right\\'>Type: <\\/td><td>" & evnttype & "<\\/td><\\/tr>"
					end if
					if evntcaus is not "" then
						set popup to popup & "<tr><td align=\\'right\\'>Cause: <\\/td><td>" & evntcaus & "<\\/td><\\/tr>"
					end if
					if evntagnc is not "" then
						set popup to popup & "<tr><td align=\\'right\\'>Agency: <\\/td><td>" & evntagnc & "<\\/td><\\/tr>"
					end if
					
					set popup to popup & "<\\/table>', STICKY, true, CLICKCLOSE, true)\" onmouseout=\"UnTip()\">&nbsp</a>" & return
					
					set filtertxt to "<span id='Filter' >" & evnttag & " " & evntname & " " & evntplac & " " & evnttype & " " & evntaddr & " " & evntcaus & " " & evntagnc & "</span>"
					
					(*
				Now we need to position the events on the chart. Due to the resolution of the chart
				and events less than 6 months apart are likely to overlap and potentially could be invisible.
				This is particularly likely with Birth/Baptism and Death/Burial/Cremation.
				There are also cases when an individual had a very short lifespan so potentially the birth and 
				death may be too close.
				
				The logic behind the following code is summarised below.
				
				If an individual only has a birth event OR a Baptism and lived for more than 6 months
				then place the indicator just inside the start of the div so that the length of the div
				is not increased.
				
				If however the lifespan was less than 6 months place the indicator just before the div 
				to avoid any conflict.
					
				If a individual has both a birth AND baptism less than 6 months apart put the birth
				just before and the baptism just after the start of the div, but for very short lifespans
				put both just before.
				
				If an individual only has a Death event OR a Burial OR a Cremation and lived for more 
				than 6 months then place the indicator just inside the end of the div so that the length
				of the div is not increased. For short lifespans put it immediately after the div.
				
				If both Death AND Burial are present, death goes just before the end and burial just
				after except short lifespans were both go just after.
				
				The following only applies to longer lifespans as children will not write wills or have 
				probate events.
				
				If there is a Will AND/OR Death AND/OR Burial AND/OR Probate within six months we could
				have up to 4 overlapping events. Aim to put one or two inside the div and one or two
				after it. Given that a will must be written before death and death occurs before burial
				and burial is likely before probate this means the div will end between the death and
				burial.
				
				*)
					
					
					--Most events are set from the left hand side of the div as a percentage
					--of the individuals lifespan. Where the individual has the same date of
					--birth and death we get a divide by zero error so just set the position to zero/
					if ddate - bdate is 0 then
						set indicator to "0%"
					else
						set indicator to ((evntSDN - bdate) / (ddate - bdate)) * 100 & "%"
					end if
					set PosInd to "left"
					
					
					
					if evnttag is "BIRT" then --dealing with BIRT
						if ddate - bdate < 180 then --less than 6 month lifespan
							if hasBapm is "N" and hasChr is "N" then --No baptism or christening
								set indicator to "-4px"
								set PosInd to "left"
							else --There is a baptism or christening
								set indicator to "-8px"
								set PosInd to "left"
							end if
						else --lifespan of more than 6 months
							set indicator to "0px"
							set PosInd to "left"
						end if
					end if
					
					
					if evnttag is in {"BAPM", "CHR"} and evntSDN < bdate + 180 then --dealing with a baptism or christening close to birth
						if ddate - bdate < 180 then --less than 6 month lifespan
							set indicator to "-4px"
							set PosInd to "left"
						else --lifespan of more than 6 months
							if hasBirt is "N" then --No birth 
								set indicator to "0px"
								set PosInd to "left"
							else --There is a birth or christening
								set indicator to "4px"
								set PosInd to "left"
							end if
						end if
					end if
					
					-- I have at least one case of a sick child baptised shortly before death
					-- but one or more of the exact dates are not know and the baptism can
					-- in some of these circumstances appear after death. If the child lived 
					-- for less than six months we should have caught that in the section 
					-- above but if not we need to make sure the baptism is displayed first
					if evnttag is in {"BAPM", "CHR"} and ddate - bdate  180 and ddate - bdate < 400 then
						if ddate - evntSDN < evntSDN - bdate then --dealing with a baptism or christening closer to death than birth
							if evntSDN = ddate and hasDeat is "N" then --dealing with individual with no death date and baptism is last known event
								set indicator to "0px"
								set PosInd to "right"
							else
								set indicator to "4px"
								set PosInd to "right"
							end if
						else
							set indicator to "0px"
							set PosInd to "left"
						end if
					end if
					
					if evnttag is "BIRT" and ddate - bdate  180 and ddate - bdate < 400 and (hasBapm is "Y" or hasChr is "Y") then --dealing with a life between 6 months and a year but with a baptism to fit in so move birth earlier to avoid clash
						set indicator to "-4px"
						set PosInd to "left"
					end if
					
					if evnttag is "DEAT" then --dealing with Death
						if ddate - bdate < 180 or (ddate - bdate  180 and ddate - bdate < 400 and (hasBapm is "Y" or hasChr is "Y")) then --less than 6 month lifespan or 6 to 12 months but with a baptism or christening
							set indicator to "-4px"
							set PosInd to "right"
						else --lifespan of more than 6 months
							set indicator to "0px"
							set PosInd to "right"
						end if
					end if
					
					if evnttag is in {"BURI", "CREM"} and evntSDN < ddate + 180 then --dealing with a burial or cremation close to death
						if ddate - bdate < 180 then --less than 6 month lifespan
							if hasDeat is "N" then --No death
								set indicator to "-4px"
								set PosInd to "right"
							else --There is a death 
								set indicator to "-8px"
								set PosInd to "right"
							end if
						else --lifespan of more than 6 months
							if ddate - bdate < 400 and (hasBapm is "Y" or hasChr is "Y") then
								set indicator to "-8px"
								set PosInd to "right"
							else
								set indicator to "-4px"
								set PosInd to "right"
							end if
						end if
					end if
					
					if evnttag is "WILL" and evntSDN > ddate - 180 then --dealing with Will close to death
						if hasDeat is "N" then --No death
							set indicator to "0px"
							set PosInd to "right"
						else --There is a death
							set indicator to "4px"
							set PosInd to "right"
						end if
					end if
					
					if evnttag is "PROB" then --dealing with Probate
						if hasBuri is "N" and hasCrem is "N" and evntSDN < ddate + 180 then --No burial or cremation but soon after death
							set indicator to "-4px"
							set PosInd to "right"
						else if (hasBuri is "Y" or hasCrem is "Y") and evntSDN < ddate + 360 then --There is a burial or cremation and we need space to display it
							set indicator to "-8px"
							set PosInd to "right"
						end if
					end if
					
					set end of unRpt to "<span id='" & evnttag & "'; style=\"position: absolute; " & PosInd & ": " & indicator & "; \">" & popup & "</span>" & filtertxt & return & return
				end if
			end repeat
		end repeat
	end tell
	return
end GetEvents


on findandreplace(textstring, oldstr, newstr)
	repeat
		set pos to (offset of oldstr in textstring)
		if pos is 0 then
			exit repeat
		end if
		set textstring to characters 1 thru (pos - 1) of textstring & newstr & characters (pos + (length of oldstr)) thru (length of textstring) of textstring as string
	end repeat
	return textstring
	
end findandreplace


on findandreplace2(textstring, oldstr, newstr)
	set tempstring to every character of textstring
	set numchar to number of items in tempstring
	repeat with char from 1 to numchar
		if item char of tempstring is oldstr then
			set item char of tempstring to newstr
		end if
	end repeat
	return tempstring as string
	
end findandreplace2

(* Start GEDitCOM II (if needed) and verify that a documenbt is open
	return true or false if at least one document is open
*)
on CheckAvailable(sName)
	tell application "GEDitCOM II"
		if versionNumber < 1.09 then
			display dialog "This script requires GEDitCOM II, Version 1.1 or newer. Please upgrade and try again" buttons {"OK"} default button "OK" with title sName
			return false
		end if
		if number of documents is 0 then
			display dialog "You have to open a document in GEDitCOM II to use this script" buttons {"OK"} default button "OK" with title sName
			return false
		end if
	end tell
	return true
end CheckAvailable
